// ***********************************************************
// Project:  Feuer LED mit Tiny24
// Software: AVR-Studio v4.18 / WinAVR-20090313
// Author:   Jens Grtler
// Erstellt: 10.01.2010
// ***********************************************************


// --- einbinden von Libs ------------------------------------
#include <avr\io.h>                          // AVR Basisdaten einbinden
#include <avr/interrupt.h>                   // Interrupts bereitstellen
#include <avr/wdt.h>                         // Watchdog Routine einbinden
// --- einbinden externer Dateien ----------------------------
#include "Pause.h"                           // Zeitschleife einbinden


// *************************************************************************************
// *   FUNKTION - 32-Bit Zufallszahlen                                                 *
// *************************************************************************************
unsigned char My_Rand() {
   static unsigned long Z;
   if (Z == 0) Z = 0x47110815;

   if (Z & 0x80000000) {
      Z<<=1;
      Z^=0x04C11DB7;
   }
   else {
      Z<<=1;
   }
   return Z;
}


// *************************************************************************************
// *   FUNKTION - 32-Bit Zufallszahlen                                                 *
// *   Generatorformel: a=(a*b+1) MOD m, mit b=134913821 und m=2^32=4294967296         *
// *************************************************************************************
unsigned char Rand32() {
   static unsigned long Y;
   Y = (Y * 134913821 + 1) % 4294967296;     // Generatorformel
   return Y;
}


// *************************************************************************************
// *   Hauptprogramm                                                                   *
// *************************************************************************************
int main(void) {                             // Beginn Hauptprogramm

   // --- Ausgnge setzen --------------------------------------------------------------
   DDRB = (1<<DDB0) | (1<<DDB1) | (1<<DDB2) | (1<<DDB3) | (1<<DDB4);    // B-Register - als Ausgang setzen
   PORTB &= ~( (1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3) | (1<<PB4) );  // Port ausschalten (0)

   // --- Timer0 / Fast-PWM an OC0A & OC0B enstellen -----------------------------------
   TCCR0A = (1<<COM0B1) | (1<<COM0A1) | (1<<WGM01) | (1<<WGM00);  // nicht invertierte PWM / Fast-PWM Mode-3
   TCCR0B = (1<<CS01);                       // Timer0 aktivieren - Vorteiler 8

   // --- Interrupt aktivieren ---------------------------------------------------------
   sei();                                    // Interrupts aktivieren

   // --- Watchdog aktivieren ----------------------------------------------------------
   wdt_enable(WDTO_2S);                      // Watchdog auf 2 Sekunden einstellen


   // **********************************************************************************
   // *   Endlosschleife                                                               *
   // **********************************************************************************
   while(1) {                                // Beginn While - Endlosschleife

      // --- Watchdog zurcksetzen -----------------------------------------------------
      wdt_reset();                           // Watchdog Reset

      // --- warten --------------------------------------------------------------------
      Pause(100);                            // 0,1 Sekunde Pause

      // --- PWM-OCR0A und PWM-OCR0B einstellen ----------------------------------------
      OCR0A = Rand32();                      // PWM mit "Rand32" Zufallszahlen laden
	  OCR0B = My_Rand();                     // PWM mit "My_Rand" Zufallszahlen laden
//	  OCR0B = 128;                           // PWM auf 50% einstellen

   }                                         // Ende While
}                                            // Ende Hauptprogramm








